--- asterisk-1.4.13/apps/app_read.c.orig	2006-09-07 12:54:36.000000000 -0600
+++ asterisk-1.4.13/apps/app_read.c	2007-10-16 19:52:49.000000000 -0600
@@ -78,7 +78,11 @@
 "  attempts   -- if greater than 1, that many attempts will be made in the \n"
 "                event no data is entered.\n"
 "  timeout    -- An integer number of seconds to wait for a digit response. If greater\n"
-"                than 0, that value will override the default timeout.\n\n"
+"                than 0, that value will override the default timeout. Can be floating point.\n"
+"This application sets the following channel variable upon completion:\n"
+"    READSTATUS - This is the status of the read operation.\n"
+"                 Possible values are:\n"
+"                 OK | ERRROR | HANGUP | INTERRUPTED | SKIPPED | TIMEOUT\n\n"
 "Read should disconnect if the function fails or errors out.\n";
 
 
@@ -94,8 +98,9 @@
 	char *argcopy = NULL;
 	struct tone_zone_sound *ts;
 	struct ast_flags flags = {0};
+	char *status = "ERROR";
 
-	 AST_DECLARE_APP_ARGS(arglist,
+	AST_DECLARE_APP_ARGS(arglist,
 		AST_APP_ARG(variable);
 		AST_APP_ARG(filename);
 		AST_APP_ARG(maxdigits);
@@ -104,6 +109,7 @@
 		AST_APP_ARG(timeout);
 	);
 	
+	pbx_builtin_setvar_helper(chan, "READSTATUS", status);
 	if (ast_strlen_zero(data)) {
 		ast_log(LOG_WARNING, "Read requires an argument (variable)\n");
 		return -1;
@@ -126,11 +132,17 @@
 	}
 
 	if (!ast_strlen_zero(arglist.timeout)) {
-		to = atoi(arglist.timeout);
-		if (to <= 0)
+		double tosec = atof(arglist.timeout);
+		if (tosec <= 0)
 			to = 0;
-		else
-			to *= 1000;
+		else {
+			to = tosec * 1000.0;
+			if (to <= 0) {
+				/* Make sure we don't use the default timeout. */
+				ast_log(LOG_WARNING, "Read timeout less than 1 ms, assuming 1 ms timeout\n");
+				to = 1;
+			}
+		}
 	}
 
 	if (ast_strlen_zero(arglist.filename)) {
@@ -158,6 +170,7 @@
 		if (ast_test_flag(&flags,OPT_SKIP)) {
 			/* At the user's option, skip if the line is not up */
 			pbx_builtin_setvar_helper(chan, arglist.variable, "\0");
+			pbx_builtin_setvar_helper(chan, "READSTATUS", "SKIPPED");
 			ast_module_user_remove(u);
 			return 0;
 		} else if (!ast_test_flag(&flags,OPT_NOANSWER)) {
@@ -176,17 +189,29 @@
 					res = ast_waitfordigit(chan, to);
 					ast_playtones_stop(chan);
 					if (res < 1) {
+						if (res == 0)
+							status = "TIMEOUT";
 						tmp[x]='\0';
 						break;
 					}
 					tmp[x++] = res;
 					if (tmp[x-1] == '#') {
 						tmp[x-1] = '\0';
+						status = "OK";
 						break;
 					}
+					if (x >= maxdigits) {
+						status = "OK";
+					}
 				}
 			} else {
 				res = ast_app_getdata(chan, arglist.filename, tmp, maxdigits, to);
+				if (res == 0)
+					status = "OK";
+				else if (res == 1)
+					status = "TIMEOUT";
+				else if (res == 2)
+					status = "INTERRUPTED";
 			}
 			if (res > -1) {
 				pbx_builtin_setvar_helper(chan, arglist.variable, tmp);
@@ -211,6 +236,9 @@
 			}
 		}
 	}
+	if (ast_test_flag(chan, AST_FLAG_ZOMBIE) || ast_check_hangup(chan))
+		status = "HANGUP";
+	pbx_builtin_setvar_helper(chan, "READSTATUS", status);
 	ast_module_user_remove(u);
 	return res;
 }
